@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -58,6 +58,9 @@ extern void script_open (void);
 extern void script_close (void);
 
 extern FILE *os_path_open (const char *, const char *);
+extern FILE *os_load_story (void);
+extern int os_storyfile_seek (FILE * fp, long offset, int whence);
+extern int os_storyfile_tell (FILE * fp);
 
 extern zword save_quetzal (FILE *, FILE *);
 extern zword restore_quetzal (FILE *, FILE *);
@@ -69,7 +72,7 @@ extern void (*op1_opcodes[]) (void);
 extern void (*op2_opcodes[]) (void);
 extern void (*var_opcodes[]) (void);
 
-char save_name[MAX_FILE_NAME + 1] = DEFAULT_SAVE_NAME;
+/* char save_name[MAX_FILE_NAME + 1] = DEFAULT_SAVE_NAME; */
 char auxilary_name[MAX_FILE_NAME + 1] = DEFAULT_AUXILARY_NAME;
 
 zbyte far *zmp = NULL;
@@ -101,13 +104,13 @@ static zbyte *undo_mem = NULL, *prev_zmp, *undo_diff;
 
 static int undo_count = 0;
 
+
 /*
  * get_header_extension
  *
  * Read a value from the header extension (former mouse table).
  *
  */
-
 zword get_header_extension (int entry)
 {
     zword addr;
@@ -117,19 +120,19 @@ zword get_header_extension (int entry)
 	return 0;
 
     addr = h_extension_table + 2 * entry;
-    LOW_WORD (addr, val)
+    LOW_WORD (addr, val);
 
     return val;
 
 }/* get_header_extension */
 
+
 /*
  * set_header_extension
  *
  * Set an entry in the header extension (former mouse table).
  *
  */
-
 void set_header_extension (int entry, zword val)
 {
     zword addr;
@@ -138,17 +141,17 @@ void set_header_extension (int entry, zword val)
 	return;
 
     addr = h_extension_table + 2 * entry;
-    SET_WORD (addr, val)
+    SET_WORD (addr, val);
 
 }/* set_header_extension */
 
+
 /*
  * restart_header
  *
  * Set all header fields which hold information about the interpreter.
  *
  */
-
 void restart_header (void)
 {
     zword screen_x_size;
@@ -158,14 +161,14 @@ void restart_header (void)
 
     int i;
 
-    SET_BYTE (H_CONFIG, h_config)
-    SET_WORD (H_FLAGS, h_flags)
+    SET_BYTE (H_CONFIG, h_config);
+    SET_WORD (H_FLAGS, h_flags);
 
     if (h_version >= V4) {
-	SET_BYTE (H_INTERPRETER_NUMBER, h_interpreter_number)
-	SET_BYTE (H_INTERPRETER_VERSION, h_interpreter_version)
-	SET_BYTE (H_SCREEN_ROWS, h_screen_rows)
-	SET_BYTE (H_SCREEN_COLS, h_screen_cols)
+	SET_BYTE (H_INTERPRETER_NUMBER, h_interpreter_number);
+	SET_BYTE (H_INTERPRETER_VERSION, h_interpreter_version);
+	SET_BYTE (H_SCREEN_ROWS, h_screen_rows);
+	SET_BYTE (H_SCREEN_COLS, h_screen_cols);
     }
 
     /* It's less trouble to use font size 1x1 for V5 games, especially
@@ -184,30 +187,30 @@ void restart_header (void)
     }
 
     if (h_version >= V5) {
-	SET_WORD (H_SCREEN_WIDTH, screen_x_size)
-	SET_WORD (H_SCREEN_HEIGHT, screen_y_size)
-	SET_BYTE (H_FONT_HEIGHT, font_y_size)
-	SET_BYTE (H_FONT_WIDTH, font_x_size)
-	SET_BYTE (H_DEFAULT_BACKGROUND, h_default_background)
-	SET_BYTE (H_DEFAULT_FOREGROUND, h_default_foreground)
+	SET_WORD (H_SCREEN_WIDTH, screen_x_size);
+	SET_WORD (H_SCREEN_HEIGHT, screen_y_size);
+	SET_BYTE (H_FONT_HEIGHT, font_y_size);
+	SET_BYTE (H_FONT_WIDTH, font_x_size);
+	SET_BYTE (H_DEFAULT_BACKGROUND, h_default_background);
+	SET_BYTE (H_DEFAULT_FOREGROUND, h_default_foreground);
     }
 
     if (h_version == V6)
 	for (i = 0; i < 8; i++)
 	    storeb ((zword) (H_USER_NAME + i), h_user_name[i]);
 
-    SET_BYTE (H_STANDARD_HIGH, h_standard_high)
-    SET_BYTE (H_STANDARD_LOW, h_standard_low)
+    SET_BYTE (H_STANDARD_HIGH, h_standard_high);
+    SET_BYTE (H_STANDARD_LOW, h_standard_low);
 
 }/* restart_header */
 
+
 /*
  * init_memory
  *
  * Allocate memory and load the story file.
  *
  */
-
 void init_memory (void)
 {
     long size;
@@ -249,8 +252,8 @@ void init_memory (void)
 
     /* Open story file */
 
-    if ((story_fp = os_path_open(story_name, "rb")) == NULL)
-	os_fatal ("Cannot open story file");
+    if ((story_fp = os_load_story()) == NULL)
+        os_fatal ("Cannot open story file");
 
     /* Allocate memory for story header */
 
@@ -264,27 +267,27 @@ void init_memory (void)
 
     /* Copy header fields to global variables */
 
-    LOW_BYTE (H_VERSION, h_version)
+    LOW_BYTE (H_VERSION, h_version);
 
     if (h_version < V1 || h_version > V8)
 	os_fatal ("Unknown Z-code version");
 
-    LOW_BYTE (H_CONFIG, h_config)
+    LOW_BYTE (H_CONFIG, h_config);
 
     if (h_version == V3 && (h_config & CONFIG_BYTE_SWAPPED))
 	os_fatal ("Byte swapped story file");
 
-    LOW_WORD (H_RELEASE, h_release)
-    LOW_WORD (H_RESIDENT_SIZE, h_resident_size)
-    LOW_WORD (H_START_PC, h_start_pc)
-    LOW_WORD (H_DICTIONARY, h_dictionary)
-    LOW_WORD (H_OBJECTS, h_objects)
-    LOW_WORD (H_GLOBALS, h_globals)
-    LOW_WORD (H_DYNAMIC_SIZE, h_dynamic_size)
-    LOW_WORD (H_FLAGS, h_flags)
+    LOW_WORD (H_RELEASE, h_release);
+    LOW_WORD (H_RESIDENT_SIZE, h_resident_size);
+    LOW_WORD (H_START_PC, h_start_pc);
+    LOW_WORD (H_DICTIONARY, h_dictionary);
+    LOW_WORD (H_OBJECTS, h_objects);
+    LOW_WORD (H_GLOBALS, h_globals);
+    LOW_WORD (H_DYNAMIC_SIZE, h_dynamic_size);
+    LOW_WORD (H_FLAGS, h_flags);
 
     for (i = 0, addr = H_SERIAL; i < 6; i++, addr++)
-	LOW_BYTE (addr, h_serial[i])
+	LOW_BYTE (addr, h_serial[i]);
 
     /* Auto-detect buggy story files that need special fixes */
 
@@ -306,8 +309,8 @@ void init_memory (void)
 
     }
 
-    LOW_WORD (H_ABBREVIATIONS, h_abbreviations)
-    LOW_WORD (H_FILE_SIZE, h_file_size)
+    LOW_WORD (H_ABBREVIATIONS, h_abbreviations);
+    LOW_WORD (H_FILE_SIZE, h_file_size);
 
     /* Calculate story file size in bytes */
 
@@ -322,18 +325,18 @@ void init_memory (void)
 
     } else {		/* some old games lack the file size entry */
 
-	fseek (story_fp, 0, SEEK_END);
-	story_size = ftell (story_fp);
-	fseek (story_fp, 64, SEEK_SET);
+	os_storyfile_seek (story_fp, 0, SEEK_END);
+	story_size = os_storyfile_tell (story_fp);
+	os_storyfile_seek (story_fp, 64, SEEK_SET);
 
     }
 
-    LOW_WORD (H_CHECKSUM, h_checksum)
-    LOW_WORD (H_ALPHABET, h_alphabet)
-    LOW_WORD (H_FUNCTIONS_OFFSET, h_functions_offset)
-    LOW_WORD (H_STRINGS_OFFSET, h_strings_offset)
-    LOW_WORD (H_TERMINATING_KEYS, h_terminating_keys)
-    LOW_WORD (H_EXTENSION_TABLE, h_extension_table)
+    LOW_WORD (H_CHECKSUM, h_checksum);
+    LOW_WORD (H_ALPHABET, h_alphabet);
+    LOW_WORD (H_FUNCTIONS_OFFSET, h_functions_offset);
+    LOW_WORD (H_STRINGS_OFFSET, h_strings_offset);
+    LOW_WORD (H_TERMINATING_KEYS, h_terminating_keys);
+    LOW_WORD (H_EXTENSION_TABLE, h_extension_table);
 
     /* Zork Zero Macintosh doesn't have the graphics flag set */
 
@@ -364,7 +367,7 @@ void init_memory (void)
 	if (story_size - size < 0x8000)
 	    n = (unsigned) (story_size - size);
 
-	SET_PC (size)
+	SET_PC (size);
 
 	if (fread (pcp, 1, n, story_fp) != n)
 	    os_fatal ("Story file read error");
@@ -378,6 +381,7 @@ void init_memory (void)
 
 }/* init_memory */
 
+
 /*
  * init_undo
  *
@@ -386,7 +390,6 @@ void init_memory (void)
  * during the game, e.g. for loading sounds or pictures.
  *
  */
-
 void init_undo (void)
 {
     void far *reserved;
@@ -413,13 +416,13 @@ void init_undo (void)
 
 }/* init_undo */
 
+
 /*
  * free_undo
  *
  * Free count undo blocks from the beginning of the undo list.
  *
  */
-
 static void free_undo (int count)
 {
     undo_t *p;
@@ -440,16 +443,16 @@ static void free_undo (int count)
 	last_undo = NULL;
 }/* free_undo */
 
+
 /*
  * reset_memory
  *
  * Close the story file and deallocate memory.
  *
  */
-
 void reset_memory (void)
 {
-    if (story_fp) 
+    if (story_fp)
 	fclose (story_fp);
     story_fp = NULL;
 
@@ -466,16 +469,15 @@ void reset_memory (void)
     zmp = NULL;
 }/* reset_memory */
 
+
 /*
  * storeb
  *
  * Write a byte value to the dynamic Z-machine memory.
  *
  */
-
 void storeb (zword addr, zbyte value)
 {
-
     if (addr >= h_dynamic_size)
 	runtime_error (ERR_STORE_RANGE);
 
@@ -496,32 +498,31 @@ void storeb (zword addr, zbyte value)
 
     }
 
-    SET_BYTE (addr, value)
+    SET_BYTE (addr, value);
 
 }/* storeb */
 
+
 /*
  * storew
  *
  * Write a word value to the dynamic Z-machine memory.
  *
  */
-
 void storew (zword addr, zword value)
 {
-
     storeb ((zword) (addr + 0), hi (value));
     storeb ((zword) (addr + 1), lo (value));
 
 }/* storew */
 
+
 /*
  * z_restart, re-load dynamic area, clear the stack and set the PC.
  *
  * 	no zargs used
  *
  */
-
 void z_restart (void)
 {
     static bool first_restart = TRUE;
@@ -534,7 +535,7 @@ void z_restart (void)
 
     if (!first_restart) {
 
-	fseek (story_fp, 0, SEEK_SET);
+	os_storyfile_seek (story_fp, 0, SEEK_SET);
 
 	if (fread (zmp, 1, h_dynamic_size, story_fp) != h_dynamic_size)
 	    os_fatal ("Story file read error");
@@ -550,7 +551,7 @@ void z_restart (void)
     if (h_version != V6) {
 
 	long pc = (long) h_start_pc;
-	SET_PC (pc)
+	SET_PC (pc);
 
     } else call (h_start_pc, 0, NULL, 0);
 
@@ -558,6 +559,7 @@ void z_restart (void)
 
 }/* z_restart */
 
+
 /*
  * get_default_name
  *
@@ -565,23 +567,21 @@ void z_restart (void)
  * copy it to a string.
  *
  */
-
 static void get_default_name (char *default_name, zword addr)
 {
-
     if (addr != 0) {
 
 	zbyte len;
 	int i;
 
-	LOW_BYTE (addr, len)
+	LOW_BYTE (addr, len);
 	addr++;
 
 	for (i = 0; i < len; i++) {
 
 	    zbyte c;
 
-	    LOW_BYTE (addr, c)
+	    LOW_BYTE (addr, c);
 	    addr++;
 
 	    if (c >= 'A' && c <= 'Z')
@@ -596,10 +596,11 @@ static void get_default_name (char *default_name, zword addr)
 	if (strchr (default_name, '.') == NULL)
 	    strcpy (default_name + i, ".AUX");
 
-    } else strcpy (default_name, auxilary_name);
+    } else strcpy (default_name, f_setup.aux_name);
 
 }/* get_default_name */
 
+
 /*
  * z_restore, restore [a part of] a Z-machine state from disk
  *
@@ -608,7 +609,6 @@ static void get_default_name (char *default_name, zword addr)
  *	zargs[2] = address of suggested file name
  *
  */
-
 void z_restore (void)
 {
     char new_name[MAX_FILE_NAME + 1];
@@ -626,7 +626,7 @@ void z_restore (void)
 	if (os_read_file_name (new_name, default_name, FILE_LOAD_AUX) == 0)
 	    goto finished;
 
-	strcpy (auxilary_name, default_name);
+	strcpy (f_setup.aux_name, default_name);
 
 	/* Open auxilary file */
 
@@ -650,70 +650,17 @@ void z_restore (void)
 
 	/* Get the file name */
 
-	if (os_read_file_name (new_name, save_name, FILE_RESTORE) == 0)
+	if (os_read_file_name (new_name, f_setup.save_name, FILE_RESTORE) == 0)
 	    goto finished;
 
-	strcpy (save_name, new_name);
+	strcpy (f_setup.save_name, new_name);
 
 	/* Open game file */
 
 	if ((gfp = fopen (new_name, "rb")) == NULL)
 	    goto finished;
 
-	if (f_setup.save_quetzal) {
-	    success = restore_quetzal (gfp, story_fp);
-
-	} else {
-	    /* Load game file */
-
-	    release = (unsigned) fgetc (gfp) << 8;
-	    release |= fgetc (gfp);
-
-	    (void) fgetc (gfp);
-	    (void) fgetc (gfp);
-
-	    /* Check the release number */
-
-	    if (release == h_release) {
-
-		pc = (long) fgetc (gfp) << 16;
-		pc |= (unsigned) fgetc (gfp) << 8;
-		pc |= fgetc (gfp);
-
-		SET_PC (pc)
-
-		sp = stack + (fgetc (gfp) << 8);
-		sp += fgetc (gfp);
-		fp = stack + (fgetc (gfp) << 8);
-		fp += fgetc (gfp);
-
-		for (i = (int) (sp - stack); i < STACK_SIZE; i++) {
-		    stack[i] = (unsigned) fgetc (gfp) << 8;
-		    stack[i] |= fgetc (gfp);
-		}
-
-		fseek (story_fp, 0, SEEK_SET);
-
-		for (addr = 0; addr < h_dynamic_size; addr++) {
-		    int skip = fgetc (gfp);
-		    for (i = 0; i < skip; i++)
-			zmp[addr++] = fgetc (story_fp);
-		    zmp[addr] = fgetc (gfp);
-		    (void) fgetc (story_fp);
-		}
-
-		/* Check for errors */
-
-		if (ferror (gfp) || ferror (story_fp) || addr != h_dynamic_size)
-		    success = -1;
-		else
-
-		    /* Success */
-
-		    success = 2;
-
-	    } else print_string ("Invalid save file\n");
-	}
+	success = restore_quetzal (gfp, story_fp);
 
 	if ((short) success >= 0) {
 
@@ -758,6 +705,7 @@ finished:
 
 }/* z_restore */
 
+
 /*
  * mem_diff
  *
@@ -768,7 +716,6 @@ finished:
  * Returns the number of bytes copied to diff.
  *
  */
-
 static long mem_diff (zbyte *a, zbyte *b, zword mem_size, zbyte *diff)
 {
     unsigned size = mem_size;
@@ -803,13 +750,13 @@ static long mem_diff (zbyte *a, zbyte *b, zword mem_size, zbyte *diff)
     return p - diff;
 }/* mem_diff */
 
+
 /*
  * mem_undiff
  *
  * Applies a quetzal-like diff to dest
  *
  */
-
 static void mem_undiff (zbyte *diff, long diff_length, zbyte *dest)
 {
     zbyte c;
@@ -839,15 +786,16 @@ static void mem_undiff (zbyte *diff, long diff_length, zbyte *dest)
     }
 }/* mem_undiff */
 
+
 /*
  * restore_undo
  *
  * This function does the dirty work for z_restore_undo.
  *
  */
-
 int restore_undo (void)
 {
+    long pc = curr_undo->pc;
 
     if (f_setup.undo_slots == 0)	/* undo feature unavailable */
 
@@ -860,7 +808,7 @@ int restore_undo (void)
     /* undo possible */
 
     memcpy (zmp, prev_zmp, h_dynamic_size);
-    SET_PC (curr_undo->pc)
+    SET_PC (pc);
     sp = stack + STACK_SIZE - curr_undo->stack_size;
     fp = stack + curr_undo->frame_offset;
     frame_count = curr_undo->frame_count;
@@ -876,20 +824,20 @@ int restore_undo (void)
 
 }/* restore_undo */
 
+
 /*
  * z_restore_undo, restore a Z-machine state from memory.
  *
  *	no zargs used
  *
  */
-
 void z_restore_undo (void)
 {
-
     store ((zword) restore_undo ());
 
 }/* z_restore_undo */
 
+
 /*
  * z_save, save [a part of] the Z-machine state to disk.
  *
@@ -898,7 +846,6 @@ void z_restore_undo (void)
  *	zargs[2] = address of suggested file name
  *
  */
-
 void z_save (void)
 {
     char new_name[MAX_FILE_NAME + 1];
@@ -916,7 +863,7 @@ void z_save (void)
 	if (os_read_file_name (new_name, default_name, FILE_SAVE_AUX) == 0)
 	    goto finished;
 
-	strcpy (auxilary_name, default_name);
+	strcpy (f_setup.aux_name, default_name);
 
 	/* Open auxilary file */
 
@@ -941,54 +888,17 @@ void z_save (void)
 
 	/* Get the file name */
 
-	if (os_read_file_name (new_name, save_name, FILE_SAVE) == 0)
+	if (os_read_file_name (new_name, f_setup.save_name, FILE_SAVE) == 0)
 	    goto finished;
 
-	strcpy (save_name, new_name);
+	strcpy (f_setup.save_name, new_name);
 
 	/* Open game file */
 
 	if ((gfp = fopen (new_name, "wb")) == NULL)
 	    goto finished;
 
-	if (f_setup.save_quetzal) {
-	    success = save_quetzal (gfp, story_fp);
-	} else {
-	    /* Write game file */
-
-	    fputc ((int) hi (h_release), gfp);
-	    fputc ((int) lo (h_release), gfp);
-	    fputc ((int) hi (h_checksum), gfp);
-	    fputc ((int) lo (h_checksum), gfp);
-
-	    GET_PC (pc)
-
-	    fputc ((int) (pc >> 16) & 0xff, gfp);
-	    fputc ((int) (pc >> 8) & 0xff, gfp);
-	    fputc ((int) (pc) & 0xff, gfp);
-
-	    nsp = (int) (sp - stack);
-	    nfp = (int) (fp - stack);
-
-	    fputc ((int) hi (nsp), gfp);
-	    fputc ((int) lo (nsp), gfp);
-	    fputc ((int) hi (nfp), gfp);
-	    fputc ((int) lo (nfp), gfp);
-
-	    for (i = nsp; i < STACK_SIZE; i++) {
-		fputc ((int) hi (stack[i]), gfp);
-		fputc ((int) lo (stack[i]), gfp);
-	    }
-
-	    fseek (story_fp, 0, SEEK_SET);
-
-	    for (addr = 0, skip = 0; addr < h_dynamic_size; addr++)
-		if (zmp[addr] != fgetc (story_fp) || skip == 255 || addr + 1 == h_dynamic_size) {
-		    fputc (skip, gfp);
-		    fputc (zmp[addr], gfp);
-		    skip = 0;
-		} else skip++;
-	}
+	success = save_quetzal (gfp, story_fp);
 
 	/* Close game file and check for errors */
 
@@ -1000,7 +910,6 @@ void z_save (void)
 	/* Success */
 
 	success = 1;
-
     }
 
 finished:
@@ -1012,18 +921,19 @@ finished:
 
 }/* z_save */
 
+
 /*
  * save_undo
  *
  * This function does the dirty work for z_save_undo.
  *
  */
-
 int save_undo (void)
 {
     long diff_size;
     zword stack_size;
     undo_t *p;
+    long pc;
 
     if (f_setup.undo_slots == 0)	/* undo feature unavailable */
 	return -1;
@@ -1053,7 +963,9 @@ int save_undo (void)
     } while (!p && undo_count);
     if (p == NULL)
 	return -1;
-    GET_PC (p->pc)
+    pc = p->pc;
+    GET_PC (pc);	/* Turbo C doesn't like seeing p->pc here */
+    p->pc = pc;
     p->frame_count = frame_count;
     p->diff_size = diff_size;
     p->stack_size = stack_size;
@@ -1075,27 +987,26 @@ int save_undo (void)
 
 }/* save_undo */
 
+
 /*
  * z_save_undo, save the current Z-machine state for a future undo.
  *
  *	no zargs used
  *
  */
-
 void z_save_undo (void)
 {
-
     store ((zword) save_undo ());
 
 }/* z_save_undo */
 
+
 /*
  * z_verify, check the story file integrity.
  *
  *	no zargs used
  *
  */
-
 void z_verify (void)
 {
     zword checksum = 0;
@@ -1103,7 +1014,8 @@ void z_verify (void)
 
     /* Sum all bytes in story file except header bytes */
 
-    fseek (story_fp, 64, SEEK_SET);
+    os_storyfile_seek (story_fp, 64, SEEK_SET);
+
 
     for (i = 64; i < story_size; i++)
 	checksum += fgetc (story_fp);
