@@ -9,7 +9,9 @@
    But since no os_* function uses it, it's safe to let the frotz core see
    this definition, but have the unix port see the curses version. */
 
-/* #include "../config.h" */
+#ifndef VERSION
+#define VERSION "2.44"
+#endif
 
 #ifndef __UNIX_PORT_FILE
 #include <signal.h>
@@ -29,9 +31,29 @@ typedef int bool;
 #include <stdio.h>
 
 
+/* typedef unsigned short zbyte; */
 typedef unsigned char zbyte;
 typedef unsigned short zword;
 
+
+/* This business turns out to interfere with the command history
+ * functions in the curses interface code.  Unicode support for DOS
+ * Frotz probably won't happen.  I'd like to get it working for Unix
+ * Frotz eventually.  The patch I was using was rather old and bugs
+ * might have crept in.  If you know how to implement Unicode for the
+ * Unix side, please let me know and please test it to make sure it
+ * doesn't mess with command history.
+ */
+/*
+#ifdef MSDOS_16BIT
+typedef unsigned char zchar;
+#else
+typedef unsigned short zchar;
+#endif
+*/
+
+typedef unsigned char zchar;
+
 enum story {
     BEYOND_ZORK,
     SHERLOCK,
@@ -43,7 +65,31 @@ enum story {
     UNKNOWN
 };
 
-typedef unsigned char zchar;
+/*** screen window ***/
+
+typedef struct {
+    zword y_pos;
+    zword x_pos;
+    zword y_size;
+    zword x_size;
+    zword y_cursor;
+    zword x_cursor;
+    zword left;
+    zword right;
+    zword nl_routine;
+    zword nl_countdown;
+    zword style;
+    zword colour;
+    zword font;
+    zword font_size;
+    zword attribute;
+    zword line_count;
+    zword true_fore;
+    zword true_back;
+} Zwindow;
+
+
+#include "setup.h"
 
 /*** Constants that may be set at compile time ***/
 
@@ -63,6 +109,17 @@ typedef unsigned char zchar;
 #define STACK_SIZE 1024
 #endif
 
+/* Assorted filename extensions */
+
+#define EXT_SAVE	".qzl"
+#define EXT_SCRIPT	".scr"
+#define EXT_BLORB	".blb"
+#define EXT_BLORB2	".zblb"
+#define EXT_BLORB3	".blorb"
+#define EXT_BLORB4	".zblorb"
+#define EXT_COMMAND	".rec"
+#define EXT_AUX		".aux"
+
 #ifndef DEFAULT_SAVE_NAME
 #define DEFAULT_SAVE_NAME "story.sav"
 #endif
@@ -148,7 +205,7 @@ typedef unsigned char zchar;
 
 #define CONFIG_TIMEDINPUT   0x80 /* Interpr supports timed input       - V4+ */
 
-#define SCRIPTING_FLAG	  0x0001 /* Outputting to transscription file  - V1+ */
+#define SCRIPTING_FLAG	  0x0001 /* Outputting to transcription file  - V1+ */
 #define FIXED_FONT_FLAG   0x0002 /* Use fixed width font               - V3+ */
 #define REFRESH_FLAG 	  0x0004 /* Refresh the screen                 - V6  */
 #define GRAPHICS_FLAG	  0x0008 /* Game wants to use graphics         - V5+ */
@@ -159,6 +216,8 @@ typedef unsigned char zchar;
 #define SOUND_FLAG	  0x0080 /* Game wants to use sound effects    - V5+ */
 #define MENU_FLAG	  0x0100 /* Game wants to use menus            - V6  */
 
+#define TRANSPARENT_FLAG  0x0001 /* Game wants to use transparency     - V6  */
+
 #define INTERP_DEFAULT 0
 #define INTERP_DEC_20 1
 #define INTERP_APPLE_IIE 2
@@ -172,6 +231,7 @@ typedef unsigned char zchar;
 #define INTERP_APPLE_IIGS 10
 #define INTERP_TANDY 11
 
+#define DEFAULT_COLOUR 1
 #define BLACK_COLOUR 2
 #define RED_COLOUR 3
 #define GREEN_COLOUR 4
@@ -276,7 +336,79 @@ extern zbyte *zmp;
 
 #endif
 
-/* A bunch of x86 assembly code previously appeared here. */
+#if defined (MSDOS_16BIT)
+extern zbyte *pcp;
+extern zbyte *zmp;
+
+#define lo(v)   ((zbyte *)&v)[0]
+#define hi(v)   ((zbyte *)&v)[1]
+
+#define SET_WORD(addr,v) asm {\
+    les bx,zmp;\
+    add bx,addr;\
+    mov ax,v;\
+    xchg al,ah;\
+    mov es:[bx],ax }
+
+#define LOW_WORD(addr,v) asm {\
+    les bx,zmp;\
+    add bx,addr;\
+    mov ax,es:[bx];\
+    xchg al,ah;\
+    mov v,ax }
+
+#define HIGH_WORD(addr,v) asm {\
+    mov bx,word ptr zmp;\
+    add bx,word ptr addr;\
+    mov al,bh;\
+    mov bh,0;\
+    mov ah,0;\
+    adc ah,byte ptr addr+2;\
+    mov cl,4;\
+    shl ax,cl;\
+    add ax,word ptr zmp+2;\
+    mov es,ax;\
+    mov ax,es:[bx];\
+    xchg al,ah;\
+    mov v,ax }
+
+#define CODE_WORD(v) asm {\
+    les bx,pcp;\
+    mov ax,es:[bx];\
+    xchg al,ah;\
+    mov v,ax;\
+    add word ptr pcp,2 }
+
+#define GET_PC(v) asm {\
+    mov bx,word ptr pcp+2;\
+    sub bx,word ptr zmp+2;\
+    mov ax,bx;\
+    mov cl,4;\
+    shl bx,cl;\
+    mov cl,12;\
+    shr ax,cl;\
+    add bx,word ptr pcp;\
+    adc al,0;\
+    sub bx,word ptr zmp;\
+    sbb al,0;\
+    mov word ptr v,bx;\
+    mov word ptr v+2,ax }
+
+#define SET_PC(v) asm {\
+    mov bx,word ptr zmp;\
+    add bx,word ptr v;\
+    mov al,bh;\
+    mov bh,0;\
+    mov ah,0;\
+    adc ah,byte ptr v+2;\
+    mov cl,4;\
+    shl ax,cl;\
+    add ax,word ptr zmp+2;\
+    mov word ptr pcp,bx;\
+    mov word ptr pcp+2,ax }
+
+#endif /* MSDOS_16BIT */
+
 
 #if !defined (AMIGA) && !defined (MSDOS_16BIT)
 
@@ -297,7 +429,6 @@ extern zbyte *zmp;
 
 
 /*** Story file header data ***/
-
 extern zbyte h_version;
 extern zbyte h_config;
 extern zword h_release;
@@ -336,11 +467,12 @@ extern zword hx_table_size;
 extern zword hx_mouse_x;
 extern zword hx_mouse_y;
 extern zword hx_unicode_table;
+extern zword hx_flags;
+extern zword hx_fore_colour;
+extern zword hx_back_colour;
 
 /*** Various data ***/
 
-extern char *story_name;
-
 extern enum story story_id;
 extern long story_size;
 
@@ -364,6 +496,8 @@ extern int mwin;
 
 extern int mouse_x;
 extern int mouse_y;
+extern int menu_selected;
+extern int mouse_button;
 
 extern bool enable_wrapping;
 extern bool enable_scripting;
@@ -376,12 +510,6 @@ extern char *option_zcode_path;	/* dg */
 extern long reserve_mem;
 
 
-/*** Blorb stuff ***/
-/*
-bb_err_t       blorb_err;
-bb_map_t       *blorb_map;
-*/
-
 /*** Z-machine opcodes ***/
 
 void 	z_add (void);
@@ -557,6 +685,10 @@ void	runtime_error (int);
 
 #define ERR_DEFAULT_REPORT_MODE ERR_REPORT_ONCE
 
+/*** Assorted initialization functions ***/
+void   init_buffer (void);
+void   init_process (void);
+void   init_sound (void);
 
 /*** Various global functions ***/
 
@@ -580,6 +712,11 @@ void 	branch (bool);
 void	storeb (zword, zbyte);
 void	storew (zword, zword);
 
+
+	/*** returns the current window ***/
+Zwindow * curwinrec( void);
+
+
 /*** Interface functions ***/
 
 void 	os_beep (int);
@@ -587,9 +724,9 @@ int  	os_char_width (zchar);
 void 	os_display_char (zchar);
 void 	os_display_string (const zchar *);
 void 	os_draw_picture (int, int, int);
-void 	os_erase_area (int, int, int, int);
-void 	os_fatal (const char *);
-void 	os_finish_with_sample (int);
+void 	os_erase_area (int, int, int, int, int);
+void 	os_fatal (const char *, ...);
+void 	os_finish_with_sample ();
 int  	os_font_data (int, int *, int *);
 void 	os_init_screen (void);
 void 	os_more_prompt (void);
@@ -609,9 +746,8 @@ void 	os_set_cursor (int, int);
 void 	os_set_font (int);
 void 	os_set_text_style (int);
 void 	os_start_sample (int, int, int, zword);
-void 	os_stop_sample (int);
+void 	os_stop_sample ();
 int  	os_string_width (const zchar *);
 void	os_init_setup (void);
 int	os_speech_output(const zchar *);
 
-#include "setup.h"
